@@ -1,4 +1,4 @@
-/* $Id: Qfb.c 13201 2011-05-21 09:03:01Z kb $
+/* $Id$
 
 Copyright (C) 2000-2005  The PARI group.
 
@@ -876,7 +876,7 @@ qfr5_init(GEN x, struct qfr_data *S)
   {
     pari_sp av=avma;
     long e;
-    S->isqrtD = grndtoi(S->sqrtD,&e);
+    S->isqrtD = gcvtoi(S->sqrtD,&e);
     if (e>-2) { avma = av; S->isqrtD = sqrti(S->D); }
   }
   else if (typ(S->isqrtD) != t_INT) pari_err(arither1);
