@@ -1,4 +1,4 @@
-/* $Id: alglin1.c 13216 2011-06-08 08:52:02Z bill $
+/* $Id$
 
 Copyright (C) 2000  The PARI group.
 
@@ -2933,7 +2933,7 @@ eigen(GEN x, long prec)
     r3 = grndtoi(r2, &e); if (e < ex) r2 = r3;
     ssesp = ker_aux(RgM_Rg_add_shallow(x, gneg(r2))); l = lg(ssesp);
     if (l == 1 || ly + (l-1) > n)
-      pari_err(syntaxer, "missing eigenspace. Compute the matrix to higher accuracy, then restart eigen at the current precision",NULL,NULL);
+      pari_err(talker, "missing eigenspace. Compute the matrix to higher accuracy, then restart eigen at the current precision");
     for (i=1; i<l; i++,ly++) gel(y,ly) = gel(ssesp,i); /* eigenspace done */
 
     r1=r2; /* try to find a different eigenvalue */
