@@ -1,4 +1,4 @@
-/* $Id: alglin2.c 13145 2011-04-20 21:56:40Z kb $
+/* $Id$
 
 Copyright (C) 2000  The PARI group.
 
@@ -777,10 +777,15 @@ gtrace(GEN x)
       }
       return gmul2n(gel(x,2),1);
 
-    case t_POL: case t_SER:
+    case t_POL:
       y = cgetg_copy(x, &lx); y[1] = x[1];
       for (i=2; i<lx; i++) gel(y,i) = gtrace(gel(x,i));
-      return y;
+      return normalizepol_lg(y, lx);
+
+    case t_SER:
+      y = cgetg_copy(x, &lx); y[1] = x[1];
+      for (i=2; i<lx; i++) gel(y,i) = gtrace(gel(x,i));
+      return normalize(y);
 
     case t_POLMOD:
       y = gel(x,1); z = gel(x,2);
