@@ -1,4 +1,4 @@
-/* $Id: arith1.c 13145 2011-04-20 21:56:40Z kb $
+/* $Id$
 
 Copyright (C) 2000  The PARI group.
 
@@ -875,11 +875,19 @@ gisanypower(GEN x, GEN *pty)
     pari_sp av = avma;
     GEN fa, P, E, a = gel(x,1), b = gel(x,2);
     long i, j, p, e;
-    int sw = (cmpii(a, b) > 0);
+    int sw = (absi_cmp(a, b) > 0);
 
     if (sw) swap(a, b);
     k = Z_isanypower(a, pty? &a: NULL);
-    if (!k) { avma = av; return 0; }
+    if (!k)
+    { /* a = -1,1 or not a pure power */
+      if (!is_pm1(a)) { avma = av; return 0; }
+      if (signe(a) < 0) b = negi(b);
+      k = Z_isanypower(b, pty? &b: NULL);
+      if (!k || !pty) { avma = av; return k; }
+      *pty = gerepileupto(av, ginv(b));
+      return k;
+    }
     fa = factoru(k);
     P = gel(fa,1);
     E = gel(fa,2); h = k;
@@ -901,11 +909,11 @@ gisanypower(GEN x, GEN *pty)
   return 0; /* not reached */
 }
 
-long
-Z_isanypower(GEN x, GEN *pty)
+/* disregard the sign of x, caller will take care of x < 0 */
+static long
+Z_isanypower_aux(GEN x, GEN *pty)
 {
-  pari_sp av = avma;
-  long ex, v, i, j, l, k, s = signe(x);
+  long ex, v, i, j, l, k;
   GEN logx, y, fa, P, E, Pe, Ee;
   byteptr d = diffptr;
   ulong mask, p = 0, ex0 = 11, e = 0, e2;
@@ -941,7 +949,7 @@ Z_isanypower(GEN x, GEN *pty)
     long v3, v5, v7, le;
     ulong e2 = e;
     v = u_lvalrem(e2, 2, &e2);
-    if (v && s > 0)
+    if (v)
     {
       for (i = 0; i < v; i++)
       {
@@ -1006,8 +1014,7 @@ Z_isanypower(GEN x, GEN *pty)
     }
   }
 END:
-  if (!pty || k == 1) avma = av;
-  else
+  if (pty && k != 1)
   {
     if (e)
     { /* add missing small factors */
@@ -1015,11 +1022,35 @@ END:
       for (i = 2; i < l; i++) y = mulii(y, powuu(P[i], E[i] / k));
       x = is_pm1(x)? y: mulii(x,y);
     }
-    if (s < 0) togglesign(x);
-    *pty = gerepilecopy(av, x);
+    *pty = x;
   }
   return k == 1? 0: k;
 }
+long
+Z_isanypower(GEN x, GEN *pty)
+{
+  pari_sp av = avma;
+  long k = Z_isanypower_aux(x, pty);
+  if (!k) { avma = av; return 0; }
+  if (signe(x) < 0)
+  {
+    long v = vals(k);
+    if (v)
+    {
+      GEN y;
+      k >>= v;
+      if (k == 1) { avma = av; return 0; }
+      if (!pty) { avma = av; return k; }
+      y = *pty;
+      y = powiu(y, 1<<v);
+      togglesign(y);
+      *pty = gerepileuptoint(av, y);
+      return k;
+    }
+  }
+  if (pty) *pty = gerepilecopy(av, *pty); else avma = av;
+  return k;
+}
 
 /*********************************************************************/
 /**                                                                 **/
