@@ -1,4 +1,4 @@
-/* $Id: bibli1.c 13145 2011-04-20 21:56:40Z kb $
+/* $Id$
 
 Copyright (C) 2000  The PARI group.
 
@@ -1813,7 +1813,6 @@ minim0(GEN a, GEN BORNE, GEN STOCKMAX, long flag)
       {
         BOUND=gtodouble(gnorme)*(1+eps); s=0;
         affii(gnorme,BORNE); avma = av1;
-        if (flag == min_PERF) invp = matid(maxrank);
       }
     }
     s++;
@@ -1848,9 +1847,16 @@ minim0(GEN a, GEN BORNE, GEN STOCKMAX, long flag)
       case min_PERF:
       {
         long I=1;
-        pari_sp av2=avma;
+        pari_sp av2;
 
-        for (i=1; i<=n; i++)
+        if (s == 1) {
+          invp = matid(maxrank);
+          for (i = 1; i <= maxrank; i++) L[i] = 0;
+        }
+        /* must go till the end in case we find a smallest vector last */
+        if (s > maxrank) { s = maxrank; continue; }
+        av2 = avma;
+        for (i = I = 1; i<=n; i++)
           for (j=i; j<=n; j++,I++) V[I] = x[i]*x[j];
         if (! addcolumntomatrix(V,invp,L))
         {
@@ -1859,12 +1865,6 @@ minim0(GEN a, GEN BORNE, GEN STOCKMAX, long flag)
         }
 
         if (DEBUGLEVEL>1) { err_printf("*"); err_flush(); }
-        if (s == maxrank)
-        {
-          if (DEBUGLEVEL>1) { err_printf("\n"); err_flush(); }
-          avma=av0; return stoi(s);
-        }
-
         if (low_stack(lim, stack_lim(av1,1)))
         {
           if(DEBUGMEM>1) pari_warn(warnmem,"minim0, rank>=%ld",s);
