@@ -1,4 +1,4 @@
-/* $Id: gen1.c 13221 2011-06-11 15:28:43Z kb $
+/* $Id$
 
 Copyright (C) 2000  The PARI group.
 
@@ -531,6 +531,8 @@ conjvec(GEN x,long prec)
         }
       }
       av = avma;
+      if (typ(x) != t_POL)
+        return gerepilecopy(av, const_col(lx-3, x));
       if (varn(x) != varn(T))
         pari_err(talker,"inconsistent variables in conjvec");
       r = cleanroots(T,prec);
@@ -2586,7 +2588,7 @@ gdiv(GEN x, GEN y)
 
         case t_INTMOD: { GEN Y = gel(y,1);
           z = cgetg(3,t_INTMOD); p1 = remii(mulii(gel(y,2),gel(x,2)), Y);
-          return div_intmod_same(z, Y, gel(x,1), p1);
+          return div_intmod_same(z, Y, modii(gel(x,1), Y), p1);
         }
 
         case t_FFELT: av=avma;
