@@ -1,4 +1,4 @@
-/* $Id: gen2.c 13218 2011-06-09 12:48:50Z kb $
+/* $Id$
 
 Copyright (C) 2000  The PARI group.
 
@@ -698,7 +698,9 @@ polidentical(GEN x, GEN y)
   for (lx--; lx >= 2; lx--) if (!gidentical(gel(x,lx), gel(y,lx))) return 0;
   return 1;
 }
-
+/* x,y t_SER */
+static int
+seridentical(GEN x, GEN y) { return polidentical(x,y); }
 /* typ(x) = typ(y) = t_VEC/COL/MAT */
 static int
 vecidentical(GEN x, GEN y)
@@ -753,7 +755,8 @@ gidentical(GEN x, GEN y)
       return gidentical(gel(x,2),gel(y,2)) && polidentical(gel(x,1),gel(y,1));
     case t_POL:
       return polidentical(x,y);
-
+    case t_SER:
+      return seridentical(x,y);
     case t_FFELT:
       return FF_equal(x,y);
 
