@@ -1,4 +1,4 @@
-/* $Id: gen3.c 13218 2011-06-09 12:48:50Z kb $
+/* $Id$
 
 Copyright (C) 2000  The PARI group.
 
@@ -1326,25 +1326,14 @@ gsubst(GEN x, long v, GEN y)
         return (ty==t_MAT)? scalarmat(gen_0,ly-1): gen_0;
 
       vx = varn(x);
+      if (varncmp(vx, v) > 0) return ty == t_MAT? scalarmat(x,ly-1): gcopy(x);
       if (varncmp(vx, v) < 0)
       {
-        if (varncmp(gvar(y), vx) > 0)
-        { /* easy special case */
-          z = cgetg(lx, t_POL); z[1] = x[1];
-          for (i=2; i<lx; i++) gel(z,i) = gsubst(gel(x,i),v,y);
-          return normalizepol_lg(z,lx);
-        }
-        /* general case */
-        av = avma; x = swap_vars(x, v);
-        return gerepileupto(av, poleval(x,y));
+        av = avma; z = cgetg(lx, t_POL); z[1] = x[1];
+        for (i=2; i<lx; i++) gel(z,i) = gsubst(gel(x,i),v,y);
+        return gerepileupto(av, poleval(z, pol_x(vx)));
       }
-      /* v <= vx */
-      if (ty!=t_MAT)
-        return varncmp(vx,v) > 0 ? gcopy(x): poleval(x,y);
-
-      if (varncmp(vx, v) > 0) return scalarmat(x,ly-1);
-      if (lx==3) return scalarmat(gel(x,2),ly-1);
-      return RgX_RgM_eval(x, y);
+      return ty == t_MAT? RgX_RgM_eval(x, y): poleval(x,y);
 
     case t_SER:
       vx = varn(x);
@@ -2677,7 +2666,7 @@ gtoser(GEN x, long v, long prec)
       return gerepileupto(av, rfractoser(x, v, prec));
 
     case t_QFR: case t_QFI: case t_VEC: case t_COL:
-      if (varncmp(gvar(x), v) < 0)
+      if (varncmp(gvar(x), v) <= 0)
         pari_err(talker,"main variable must have higher priority in gtoser");
       lx = lg(x); if (tx == t_QFR) lx--;
       for (i=1; i < lx; i++)
