@@ -1,4 +1,4 @@
-/* $Id: prime.c 13154 2011-04-22 20:38:44Z kb $
+/* $Id$
 
 Copyright (C) 2000  The PARI group.
 
@@ -350,6 +350,7 @@ u_IsLucasPsP(ulong n)
 static int
 IsLucasPsP(GEN N)
 {
+  pari_sp av = avma, lim = stack_lim(av, 1);
   GEN N_2, m, z;
   long i, v;
   ulong b;
@@ -370,6 +371,11 @@ IsLucasPsP(GEN N)
     if (!signe(z)) return 1;
     z = modii(subis(sqri(z), 2), N);
     if (equaliu(z, 2)) return 0;
+    if (low_stack(lim,stack_lim(av,1)))
+    {
+      if(DEBUGMEM>1) pari_warn(warnmem,"IsLucasPsP");
+      z = gerepileupto(av, z);
+    }
   }
   return 0;
 }
@@ -759,10 +765,10 @@ ulong
 uprimepi(ulong n)
 {
   byteptr p = diffptr+1;
-  ulong prime = 2, res = 1;
+  ulong prime = 2, res = 0;
   maxprime_check(n);
-  while (prime <= n) { res++; NEXT_PRIME_VIADIFF(prime,p); }
-  return res-1;
+  while (prime < n) { res++; NEXT_PRIME_VIADIFF(prime,p); }
+  return prime == n? res+1: res;
 }
 
 GEN
