@@ -1,4 +1,4 @@
-/* $Id: trans2.c 13228 2011-06-19 22:27:43Z kb $
+/* $Id$
 
 Copyright (C) 2000  The PARI group.
 
@@ -786,18 +786,27 @@ mpbern(long nb, long prec)
     set_bern(c0, 1, divru(real_1(prec), 6)); /* B2 = 1/6 */
     i = 2;
   }
+  /* B_{2i} = (2i-1) / (4i+2) -
+   * sum_{a = 1}^{i-1} (2i)...(2i+2-2a) / (2...(2a-1)2a) B_{2a} */
   for (   ; i <= nb; i++, avma = av)
   { /* i > 1 */
-    long n = 8, m = 5, d1 = i-1, d2 = 2*i-3;
-    GEN S = BERN(d1);
+#ifdef LONG_IS_64BIT
+    const ulong mul_overflow = 3037000500;
+#else
+    const ulong mul_overflow = 46341;
+#endif
+    ulong u = 8, v = 5, a = i-1, b = 2*i-3;
+    GEN S = BERN(a);
 
     for (;;)
-    {
-      S = divru(mulru(S, n*m), d1*d2);
-      if (d1 == 1) break;
-      n += 4; m += 2; d1--; d2 -= 2;
-      S = addrr(BERN(d1), S);
-      if ((d1 & 127) == 0) { set_bern(c0, i, S); S = BERN(i); avma = av; }
+    { /* b = 2a-1, u = 2v-2, 2a + v = 2i+3 */
+      if (a == 1) { S = mulri(S, muluu(u,v)); break; } /* a=b=1, v=2i+1, u=4i */
+      /* beware overflow */
+      S = (v <= mul_overflow)? mulru(S, u*v): mulri(S, muluu(u,v));
+      S = (a <= mul_overflow)? divru(S, a*b): divri(S, muluu(a,b));
+      u += 4; v += 2; a--; b -= 2;
+      S = addrr(BERN(a), S);
+      if ((a & 127) == 0) { set_bern(c0, i, S); S = BERN(i); avma = av; }
     }
     S = divru(subsr(2*i, S), 2*i+1);
     setexpo(S, expo(S) - 2*i);
