@@ -1,4 +1,4 @@
-/* $Id: gp.c 13217 2011-06-09 12:48:50Z kb $
+/* $Id$
 
 Copyright (C) 2000  The PARI group.
 
@@ -376,7 +376,8 @@ download them from the PARI WWW site 'http://pari.math.u-bordeaux.fr/'."
     suggestions, technical questions, bug reports and patch submissions.\n\
   - pari-users for everything else!\n");
   print_text("\
-To subscribe, send an empty message to <listname>-subscribe@list.cr.yp.to. \
+To subscribe, send an empty message to <listname>-request@pari.math.u-bordeaux.fr, \
+with a Subject: field containing the word \"subscribe\".  \
 An archive is kept at the WWW site mentioned above. You can also reach the \
 authors directly by email: pari@math.u-bordeaux.fr (answer not guaranteed)."); }
 
@@ -773,7 +774,7 @@ what_readline(void)
             (GP_DATA->use_readline)? "enabled": "disabled",
             extra);
 #else
-  s = "not compiled in";
+  s = (char*)"not compiled in";
 #endif
   return s;
 }
