@@ -1,4 +1,4 @@
-/* $Id: plotport.c 12886 2011-01-30 23:04:50Z bill $
+/* $Id$
 
 Copyright (C) 2000  The PARI group.
 
@@ -1982,7 +1982,18 @@ void
 rectdraw_flag(GEN list, long flag) { gendraw(list, 0, flag); }
 
 static void
-ps_sc(void *data, long col) { (void)data; (void)col; }
+ps_sc(void *data, long col)
+{
+  long l = lg(pari_colormap)-1;
+  int r, g, b;
+  if (col >= l)
+  {
+    pari_warn(warner,"non-existent color: %ld", col);
+    col = l-1;
+  }
+  color_to_rgb(gel(pari_colormap,col+1), &r, &g, &b);
+  fprintf((FILE*)data,"%f %f %f setrgbcolor\n", r/255., g/255., b/255.);
+}
 
 static void
 ps_point(void *data, long x, long y)
@@ -2883,7 +2894,7 @@ color_to_rgb(GEN c, int *r, int *g, int *b)
     case t_STR:
       colorname_to_rgb(GSTR(c), r,g,b);
       break;
-    case t_VECSMALL:
+    default: /* t_VECSMALL: */
       *r = c[1]; *g = c[2]; *b = c[3];
       break;
   }
