@@ -1,4 +1,4 @@
-/* $Id: pariinl.h 13108 2011-04-15 16:45:17Z kb $
+/* $Id$
 
 Copyright (C) 2000-2010  The PARI group.
 
@@ -554,12 +554,12 @@ RgM_is_ZM(GEN x)
 /**                                                                **/
 /********************************************************************/
 INLINE void **
-stack_base(pari_stack *s) { return (void **) ((char *)s+s->offset); }
+stack_base(pari_stack *s) { return (void **) ((ulong)s + (ulong)s->offset); }
 
 INLINE void
 stack_init(pari_stack *s, size_t size, void **data)
 {
-  s->offset = (char *)data-(char *)s;
+  s->offset = (ulong)data-(ulong)s;
   *data = NULL;
   s->n = 0;
   s->alloc = 0;
@@ -1724,13 +1724,13 @@ INLINE ulong
 F2v_coeff(GEN x,long v) { return F2x_coeff(x,v-1); }
 
 INLINE void
-F2v_clear(GEN x,long v) { return F2x_clear(x,v-1); }
+F2v_clear(GEN x,long v) { F2x_clear(x,v-1); }
 
 INLINE void
-F2v_set(GEN x,long v)   { return F2x_set(x,v-1); }
+F2v_set(GEN x,long v)   { F2x_set(x,v-1); }
 
 INLINE void
-F2v_flip(GEN x,long v) {return F2x_flip(x,v-1); }
+F2v_flip(GEN x,long v) { F2x_flip(x,v-1); }
 
 /* F2m */
 
