@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/mp.c"
-/* $Id: mp.c 13142 2011-04-20 15:17:30Z bill $
+/* $Id$
 
 Copyright (C) 2000-2003 The PARI group.
 
@@ -1636,14 +1636,14 @@ muliispec(GEN a, GEN b, long na, long nb)
   return gerepileuptoint(av, addshiftw(c,c0, n0));
 }
 
-/* x % (2^n), assuming x, n >= 0 */
+/* x % (2^n), assuming n >= 0 */
 GEN
 remi2n(GEN x, long n)
 {
-  long hi,l,k,lx,ly;
+  long hi,l,k,lx,ly, sx = signe(x);
   GEN z, xd, zd;
 
-  if (!signe(x) || !n) return gen_0;
+  if (!sx || !n) return gen_0;
 
   k = dvmdsBIL(n, &l);
   lx = lgefint(x);
@@ -1663,7 +1663,7 @@ remi2n(GEN x, long n)
     ly = k+3;
 
   zd = z = cgeti(ly);
-  *++zd = evalsigne(1) | evallgefint(ly);
+  *++zd = evalsigne(sx) | evallgefint(ly);
   if (hi) *++zd = hi;
   for ( ;k; k--) *++zd = *++xd;
   return z;
