@@ -634,7 +634,7 @@ closure_err(void)
     if ((i==lastfun || lg(trace[i+1].closure)>=7))
     {
       /* After a SIGINT, pc can be slightly off: ensure 0 <= pc < lg() */
-      long pc = trace[i].pc ? minss(*trace[i].pc, lg(mael(C,5,1))-1): 1;
+      long pc = minss(lg(mael(C,5,1))-1, trace[i].pc ? *trace[i].pc: 1);
       long offset = pc? mael3(C,5,1,pc): 0;
       int member;
       const char *s, *sbase;
@@ -666,7 +666,7 @@ closure_context(long start)
   const long lastfun = s_trace.n - 1;
   long i, fun = lastfun;
   if (fun<0) return lastfun;
-  while (fun>start+1 && lg(trace[fun].closure)==6) fun--;
+  while (fun>start && lg(trace[fun].closure)==6) fun--;
   for (i=fun; i<=lastfun; i++)
   {
     long *pc=trace[i].pc;
@@ -1316,6 +1316,7 @@ evalstate_reset(void)
   s_trace.n = 0;
   reset_break();
   compilestate_reset();
+  parsestate_reset();
   avma = top;
 }
 
